@@ -1,86 +1,100 @@
 from asyncio import (
     CancelledError,
     Task,
-    TaskGroup,
     current_task,
 )
 from collections.abc import (
     AsyncGenerator,
     AsyncIterator,
     Callable,
+    Collection,
     Coroutine,
-    Iterable,
     Mapping,
 )
+from contextlib import AbstractAsyncContextManager, AbstractContextManager
 from logging import Logger
 from types import TracebackType
 from typing import Any, final, overload
 
 from haiway.context.disposables import Disposable, Disposables
+from haiway.context.events import EventsContext, EventSubscription
 from haiway.context.identifier import ScopeIdentifier
 from haiway.context.observability import (
     Observability,
     ObservabilityAttribute,
     ObservabilityContext,
     ObservabilityLevel,
+    ObservabilityMetricKind,
+)
+
+# Import after other imports to avoid circular dependencies
+from haiway.context.presets import (
+    ContextPreset,
+    ContextPresetRegistryContext,
 )
 from haiway.context.state import ScopeState, StateContext
 from haiway.context.tasks import TaskGroupContext
-from haiway.state import State
+from haiway.context.variables import VariablesContext
+from haiway.state import Immutable, State
+from haiway.utils.collections import as_list
 from haiway.utils.stream import AsyncStream
 
 __all__ = ("ctx",)
 
 
-@final
-class ScopeContext:
-    """
-    Context manager for executing code within a defined scope.
-
-    ScopeContext manages scope-related data and behavior including identity, state,
-    observability, and task coordination. It enforces immutability and provides both
-    synchronous and asynchronous context management interfaces.
-
-    This class should not be instantiated directly; use the ctx.scope() factory method
-    to create scope contexts.
-    """
-
-    __slots__ = (
-        "_disposables",
-        "_identifier",
-        "_observability_context",
-        "_state_context",
-        "_task_group_context",
-    )
+class ScopeContext(Immutable):
+    _identifier: ScopeIdentifier
+    _state: Collection[State]
+    _state_context: StateContext | None
+    _disposables: Disposables | None
+    _preset: ContextPreset | None
+    _preset_disposables: Disposables | None
+    _observability_context: ObservabilityContext
+    _task_group_context: TaskGroupContext | None
+    _events_context: EventsContext | None
+    _variables_context: VariablesContext
 
     def __init__(
         self,
-        label: str,
-        task_group: TaskGroup | None,
+        name: str,
         state: tuple[State, ...],
+        preset: ContextPreset | None,
         disposables: Disposables | None,
         observability: Observability | Logger | None,
+        isolated: bool,
     ) -> None:
-        self._identifier: ScopeIdentifier
         object.__setattr__(
             self,
             "_identifier",
-            ScopeIdentifier.scope(label),
+            ScopeIdentifier.scope(name),
         )
-        # prepare state context to capture current state
-        self._state_context: StateContext
+        # store explicit state separately for priority control
+        object.__setattr__(
+            self,
+            "_state",
+            state,
+        )
+        # placeholder for temporary, resolved state context
         object.__setattr__(
             self,
             "_state_context",
-            StateContext.updated(state),
+            None,
         )
-        self._disposables: Disposables | None
         object.__setattr__(
             self,
             "_disposables",
             disposables,
         )
-        self._observability_context: ObservabilityContext
+        object.__setattr__(
+            self,
+            "_preset",
+            preset if preset is not None else ContextPresetRegistryContext.select(name),
+        )
+        object.__setattr__(
+            self,
+            "_preset_disposables",
+            None,
+        )
         object.__setattr__(
             self,
             "_observability_context",
@@ -90,93 +104,70 @@ class ScopeContext:
                 observability=observability,
             ),
         )
-        self._task_group_context: TaskGroupContext | None
         object.__setattr__(
             self,
             "_task_group_context",
-            TaskGroupContext(task_group=task_group)
-            if task_group is not None or self._identifier.is_root
-            else None,
+            TaskGroupContext() if self._identifier.is_root or isolated else None,
         )
-
-    def __setattr__(
-        self,
-        name: str,
-        value: Any,
-    ) -> Any:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be modified"
+        object.__setattr__(
+            self,
+            "_events_context",
+            EventsContext() if self._identifier.is_root else None,
         )
-
-    def __delattr__(
-        self,
-        name: str,
-    ) -> None:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be deleted"
-        )
-
-    def __enter__(self) -> str:
-        assert (  # nosec: B101
-            self._task_group_context is None or self._identifier.is_root
-        ), "Can't enter synchronous context with task group"
-        assert self._disposables is None, "Can't enter synchronous context with disposables"  # nosec: B101
-        self._identifier.__enter__()
-        self._observability_context.__enter__()
-        self._state_context.__enter__()
-
-        return self._observability_context.observability.trace_identifying(self._identifier).hex
-
-    def __exit__(
-        self,
-        exc_type: type[BaseException] | None,
-        exc_val: BaseException | None,
-        exc_tb: TracebackType | None,
-    ) -> None:
-        self._state_context.__exit__(
-            exc_type=exc_type,
-            exc_val=exc_val,
-            exc_tb=exc_tb,
-        )
-        self._observability_context.__exit__(
-            exc_type=exc_type,
-            exc_val=exc_val,
-            exc_tb=exc_tb,
-        )
-        self._identifier.__exit__(
-            exc_type=exc_type,
-            exc_val=exc_val,
-            exc_tb=exc_tb,
+        object.__setattr__(
+            self,
+            "_variables_context",
+            VariablesContext(isolated=self._identifier.is_root or isolated),
         )
 
     async def __aenter__(self) -> str:
+        assert self._preset_disposables is None  # nosec: B101
+        assert self._state_context is None  # nosec: B101
+
         self._identifier.__enter__()
         self._observability_context.__enter__()
 
-        if task_group := self._task_group_context:
-            await task_group.__aenter__()
+        if self._task_group_context is not None:
+            await self._task_group_context.__aenter__()
 
-        # lazily initialize state to include disposables results
-        if disposables := self._disposables:
-            assert self._state_context._token is None  # nosec: B101
+        if self._events_context is not None:
+            await self._events_context.__aenter__()
+
+        self._variables_context.__enter__()
+
+        # Collect all state sources in priority order (lowest to highest priority)
+        # 1. Add contextual state first (lowest priority)
+        collected_state: list[State] = as_list(StateContext.current_state())
+
+        # 2. Add preset state (low priority, overrides contextual)
+        if self._preset is not None:
+            preset_disposables: Disposables = await self._preset.prepare()
             object.__setattr__(
                 self,
-                "_state_context",
-                StateContext(
-                    state=ScopeState(
-                        (
-                            *self._state_context._state._state.values(),
-                            *await disposables.prepare(),
-                        )
-                    ),
-                ),
+                "_preset_disposables",
+                preset_disposables,
             )
+            collected_state.extend(await preset_disposables.prepare())
 
-        self._state_context.__enter__()
+        # 3. Add explicit disposables state (medium priority)
+        if self._disposables is not None:
+            collected_state.extend(await self._disposables.prepare())
 
-        return self._observability_context.observability.trace_identifying(self._identifier).hex
+        # 4. Add explicit state last (highest priority)
+        collected_state.extend(self._state)
+        # Create resolved state context with all collected state
+        resolved_state_context: StateContext = StateContext(
+            _state=ScopeState(tuple(collected_state))
+        )
+
+        resolved_state_context.__enter__()
+        object.__setattr__(
+            self,
+            "_state_context",
+            resolved_state_context,
+        )
+
+        return str(self._observability_context.observability.trace_identifying(self._identifier))
 
     async def __aexit__(
         self,
@@ -184,32 +175,62 @@ class ScopeContext:
         exc_val: BaseException | None,
         exc_tb: TracebackType | None,
     ) -> None:
-        if disposables := self._disposables:
-            await disposables.dispose(
+        assert self._state_context is not None  # nosec: B101
+
+        if self._disposables is not None:
+            await self._disposables.dispose(
                 exc_type=exc_type,
                 exc_val=exc_val,
                 exc_tb=exc_tb,
             )
 
-        if task_group := self._task_group_context:
-            await task_group.__aexit__(
+        if self._preset_disposables is not None:
+            await self._preset_disposables.dispose(
                 exc_type=exc_type,
                 exc_val=exc_val,
                 exc_tb=exc_tb,
             )
+            object.__setattr__(
+                self,
+                "_preset_disposables",
+                None,
+            )
+
+        if self._task_group_context is not None:
+            await self._task_group_context.__aexit__(
+                exc_type=exc_type,
+                exc_val=exc_val,
+                exc_tb=exc_tb,
+            )
+
+        if self._events_context is not None:
+            await self._events_context.__aexit__(
+                exc_type=exc_type,
+                exc_val=exc_val,
+                exc_tb=exc_tb,
+            )
+
+        self._variables_context.__exit__(
+            exc_type=exc_type,
+            exc_val=exc_val,
+            exc_tb=exc_tb,
+        )
 
         self._state_context.__exit__(
             exc_type=exc_type,
             exc_val=exc_val,
             exc_tb=exc_tb,
         )
-
+        object.__setattr__(
+            self,
+            "_state_context",
+            None,
+        )
         self._observability_context.__exit__(
             exc_type=exc_type,
             exc_val=exc_val,
             exc_tb=exc_tb,
         )
-
         self._identifier.__exit__(
             exc_type=exc_type,
             exc_val=exc_val,
@@ -217,6 +238,59 @@ class ScopeContext:
         )
 
 
+class DisposablesContext(Immutable):
+    _disposables: Disposables
+    _state_context: StateContext | None
+
+    def __init__(
+        self,
+        disposables: Disposables,
+    ) -> None:
+        object.__setattr__(
+            self,
+            "_disposables",
+            disposables,
+        )
+        object.__setattr__(
+            self,
+            "_state_context",
+            None,
+        )
+
+    async def __aenter__(self) -> None:
+        assert self._state_context is None  # nosec: B101
+        state_context: StateContext = StateContext.updated(await self._disposables.prepare())
+        state_context.__enter__()
+        object.__setattr__(
+            self,
+            "_state_context",
+            state_context,
+        )
+
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
+        assert self._state_context is not None  # nosec: B101
+        await self._disposables.dispose(
+            exc_type=exc_type,
+            exc_val=exc_val,
+            exc_tb=exc_tb,
+        )
+        self._state_context.__exit__(
+            exc_type=exc_type,
+            exc_val=exc_val,
+            exc_tb=exc_tb,
+        )
+        object.__setattr__(
+            self,
+            "_state_context",
+            None,
+        )
+
+
 @final
 class ctx:
     """
@@ -259,46 +333,190 @@ class ctx:
         return ObservabilityContext.trace_id(scope_identifier)
 
     @staticmethod
-    def scope(
-        label: str,
-        /,
-        *state: State | None,
-        disposables: Disposables | Iterable[Disposable] | None = None,
-        task_group: TaskGroup | None = None,
-        observability: Observability | Logger | None = None,
-    ) -> ScopeContext:
+    def presets(
+        *presets: ContextPreset,
+    ) -> AbstractContextManager[None]:
         """
-        Prepare scope context with given parameters. When called within an existing context\
-         it becomes nested with current context as its parent.
+        Create a context manager for a preset registry.
+
+        This method creates a registry of context presets that can be used within
+        nested scopes. Presets allow you to define reusable combinations of state
+        and disposables that can be referenced by name when creating scopes.
+
+        When entering this context manager, the provided presets become available
+        for use with ctx.scope(). The presets are looked up by their name when
+        creating scopes.
+
+        Note: For single preset usage, consider passing the preset directly to
+        ctx.scope() using the preset parameter instead of using this registry.
+        Presets only work with async contexts.
 
         Parameters
         ----------
-        label: str
-            name of the scope context
-
-        *state: State | None
-            state propagated within the scope context, will be merged with current state by\
-             replacing current with provided on conflict.
-
-        disposables: Disposables | Iterable[Disposable] | None
-            disposables consumed within the context when entered. Produced state will automatically\
-             be added to the scope state. Using asynchronous context is required if any disposables\
-             were provided.
-
-        task_group: TaskGroup | None
-            task group used for spawning and joining tasks within the context. Root scope will
-             always have task group created even when not set.
-
-        observability: Observability | Logger | None = None
-            observability solution responsible for recording and storing metrics, logs and events.\
-             Assigning observability within existing context will result in an error.
-            When not provided, logger with the scope name will be requested and used.
+        *presets: ContextPreset
+            Variable number of preset configurations to register. Each preset
+            must have a unique name within the registry.
 
         Returns
         -------
-        ScopeContext
-            context object intended to enter context manager with.\
-             context manager will provide trace_id of current context.
+        AbstractContextManager[None]
+            A context manager that makes the presets available in nested scopes
+
+        Examples
+        --------
+        Basic preset usage:
+
+        >>> from haiway import ctx, State
+        >>> from haiway.context import ContextPreset
+        >>>
+        >>> class ApiConfig(State):
+        ...     base_url: str
+        ...     timeout: int = 30
+        >>>
+        >>> # Define presets
+        >>> dev_preset = ContextPreset(
+        ...     name="development",
+        ...     _state=[ApiConfig(base_url="https://dev-api.example.com")]
+        ... )
+        >>>
+        >>> prod_preset = ContextPreset(
+        ...     name="production",
+        ...     _state=[ApiConfig(base_url="https://api.example.com", timeout=60)]
+        ... )
+        >>>
+        >>> # Use presets
+        >>> with ctx.presets(dev_preset, prod_preset):
+        ...     async with ctx.scope("development"):
+        ...         config = ctx.state(ApiConfig)
+        ...         assert config.base_url == "https://dev-api.example.com"
+
+        Nested preset registries:
+
+        >>> base_presets = [dev_preset, prod_preset]
+        >>> override_preset = ContextPreset(
+        ...     name="development",
+        ...     _state=[ApiConfig(base_url="https://staging.example.com")]
+        ... )
+        >>>
+        >>> with ctx.presets(*base_presets):
+        ...     # Outer registry has dev and prod presets
+        ...     with ctx.presets(override_preset):
+        ...         # Inner registry overrides dev preset
+        ...         async with ctx.scope("development"):
+        ...             config = ctx.state(ApiConfig)
+        ...             assert config.base_url == "https://staging.example.com"
+
+        See Also
+        --------
+        ContextPreset : For creating individual preset configurations
+        ctx.scope : For creating scopes that can use presets
+        """
+        return ContextPresetRegistryContext(presets=presets)
+
+    @staticmethod
+    def scope(
+        name: str,
+        /,
+        *state: State | None,
+        preset: ContextPreset | None = None,
+        disposables: Disposables | Collection[Disposable] | None = None,
+        observability: Observability | Logger | None = None,
+        isolated: bool = False,
+    ) -> AbstractAsyncContextManager[str]:
+        """
+        Prepare scope context with given parameters.
+
+        When called within an existing context, it becomes nested with current context
+        as its parent.
+
+        Note: Presets can only be used with async contexts. Synchronous contexts
+        do not support preset functionality.
+
+        State Priority System
+        ---------------------
+        State resolution follows a 4-layer priority system (highest to lowest):
+
+        1. **Explicit state** (passed to ctx.scope()) - HIGHEST priority
+        2. **Explicit disposables** (passed to ctx.scope()) - medium priority
+        3. **Preset state** (from presets) - low priority
+        4. **Contextual state** (from parent contexts) - LOWEST priority
+
+        When state types conflict, higher priority sources override lower priority ones.
+        State objects are resolved by type, with the highest priority instance winning.
+
+        Parameters
+        ----------
+        name: str
+            name of the scope context, can be associated with state presets
+
+        *state: State | None
+            state propagated within the scope context, will be merged with current state by
+            replacing current with provided on conflict.
+
+        preset: ContextPreset | None = None
+            context preset to be used within the scope context. The preset's state and
+            disposables will be applied to the scope with lower priority than explicit state.
+            Only works with async contexts.
+
+        disposables: Disposables | Collection[Disposable] | None
+            disposables consumed within the context when entered. Produced state will automatically
+            be added to the scope state. Using asynchronous context is required if any disposables
+            were provided.
+
+        observability: Observability | Logger | None = None
+            observability solution responsible for recording and storing metrics, logs and events.
+            Assigning observability within existing context will result in an error.
+            When not provided, logger with the scope name will be requested and used.
+
+        isolated: bool = False
+            control if scope variables and task groupd will be isolated from parent.
+            When set to True, context will use separate TaskGroup and not propagate its variables
+            to the parent scope. Isolation do not affect events propagation within the context.
+            Root scope is always isolated.
+
+        Returns
+        -------
+        AbstractAsyncContextManager[str]
+            context manager object intended to enter the scope with.
+            context manager will provide trace_id of current scope.
+
+        Examples
+        --------
+        Using a preset directly:
+
+        >>> from haiway import ctx, State
+        >>> from haiway.context import ContextPreset
+        >>>
+        >>> class ApiConfig(State):
+        ...     base_url: str
+        ...     timeout: int = 30
+        >>>
+        >>> api_preset = ContextPreset(
+        ...     name="api",
+        ...     state=[ApiConfig(base_url="https://api.example.com")]
+        ... )
+        >>>
+        >>> # Direct preset usage
+        >>> async with ctx.scope("main", preset=api_preset):
+        ...     config = ctx.state(ApiConfig)
+        ...     # Uses preset configuration
+        >>>
+        >>> # Override preset state with explicit state
+        >>> async with ctx.scope("main", ApiConfig(timeout=60), preset=api_preset):
+        ...     config = ctx.state(ApiConfig)
+        ...     # base_url from preset, timeout overridden to 60
+
+        Using preset registry (original approach):
+
+        >>> # Multiple presets registered
+        >>> with ctx.presets(dev_preset, prod_preset):
+        ...     async with ctx.scope("development"):  # Matches dev_preset by name
+        ...         config = ctx.state(ApiConfig)
+
+        See Also
+        --------
+        ctx.presets : For registering multiple presets by name
+        ContextPreset : For creating preset configurations
         """
 
         resolved_disposables: Disposables | None
@@ -309,35 +527,38 @@ class ctx:
             case Disposables() as disposables:
                 resolved_disposables = disposables
 
-            case iterable:
-                resolved_disposables = Disposables(*iterable)
+            case collection:
+                resolved_disposables = Disposables(collection)
 
         return ScopeContext(
-            label=label,
-            task_group=task_group,
+            name=name,
             state=tuple(element for element in state if element is not None),
+            preset=preset,
             disposables=resolved_disposables,
             observability=observability,
+            isolated=isolated,
         )
 
     @staticmethod
     def updated(
         *state: State | None,
-    ) -> StateContext:
+    ) -> AbstractContextManager[None]:
         """
-        Update scope context with given state. When called within an existing context\
-         it becomes nested with current context as its predecessor.
+        Update scope context with given state.
+
+        When called within an existing context, it becomes nested with current
+        context as its predecessor.
 
         Parameters
         ----------
         *state: State | None
-            state propagated within the updated scope context, will be merged with current if any\
-             by replacing current with provided on conflict
+            state propagated within the updated scope context, will be merged with current if any
+            by replacing current with provided on conflict
 
         Returns
         -------
-        StateContext
-            state part of context object intended to enter context manager with it
+        AbstractContextManager[None]
+            context manager object intended to enter updated state context with it
         """
 
         return StateContext.updated(element for element in state if element is not None)
@@ -345,7 +566,7 @@ class ctx:
     @staticmethod
     def disposables(
         *disposables: Disposable | None,
-    ) -> Disposables:
+    ) -> AbstractAsyncContextManager[None]:
         """
         Create a container for managing multiple disposable resources.
 
@@ -362,9 +583,9 @@ class ctx:
 
         Returns
         -------
-        Disposables
-            A container that manages the lifecycle of all provided disposables
-            and propagates their state to the context when used with ctx.scope()
+        AbstractAsyncContextManager[None]
+            A context manager that manages the lifecycle of all provided disposables
+            and propagates their state to the context, similar to ctx.scope()
 
         Examples
         --------
@@ -373,16 +594,19 @@ class ctx:
         >>> from haiway import ctx
         >>> async def main():
         ...
-        ...     async with ctx.scope(
-        ...         "database_work",
-        ...         disposables=(database_connection(),)
+        ...     async with ctx.disposables(
+        ...         database_connection(),
         ...     ):
         ...         # ConnectionState is now available in context
         ...         conn_state = ctx.state(ConnectionState)
         ...         await conn_state.connection.execute("SELECT 1")
         """
 
-        return Disposables(*(disposable for disposable in disposables if disposable is not None))
+        return DisposablesContext(
+            disposables=Disposables(
+                tuple(disposable for disposable in disposables if disposable is not None)
+            )
+        )
 
     @staticmethod
     def spawn[Result, **Arguments](
@@ -392,8 +616,9 @@ class ctx:
         **kwargs: Arguments.kwargs,
     ) -> Task[Result]:
         """
-        Spawn an async task within current scope context task group. When called outside of context\
-         it will spawn detached task instead.
+        Spawn an async task within current scope context task group.
+
+        When called outside of context, it will spawn detached task instead.
 
         Parameters
         ----------
@@ -442,7 +667,7 @@ class ctx:
         """
 
         output_stream = AsyncStream[Element]()
-        stream_scope: ScopeContext = ctx.scope("stream")
+        stream_scope: AbstractAsyncContextManager[str] = ctx.scope("stream")
 
         async def stream() -> None:
             async with stream_scope:
@@ -529,7 +754,7 @@ class ctx:
 
     @staticmethod
     def state[StateType: State](
-        state: type[StateType],
+        state: type[StateType] | StateType,
         /,
         default: StateType | None = None,
     ) -> StateType:
@@ -542,8 +767,9 @@ class ctx:
 
         Parameters
         ----------
-        state: type[StateType]
-            The State class type to retrieve from the current context
+        state: type[StateType] | StateType
+            The State class type to retrieve from the current context, or an
+            instance to return directly
         default: StateType | None, default=None
             Optional default instance to return if state is not found in context.
             If None and no state is found, a new instance will be created if possible.
@@ -589,11 +815,248 @@ class ctx:
         ...         config = ctx.state(DatabaseConfig)
         ...         # Use config to connect to database
         """
+        if not isinstance(state, type):
+            return state  # if we got an instance instead of type return it
+
         return StateContext.state(
             state,
             default=default,
         )
 
+    @staticmethod
+    def send(
+        payload: State,
+    ) -> None:
+        """
+        Send an event to all active subscribers within the current context.
+
+        Events are dispatched based on their exact type - subscribers must
+        subscribe to the specific State type to receive events.
+
+        Parameters
+        ----------
+        payload : State
+            The event payload to send. Must be a State instance.
+
+        Raises
+        ------
+        MissingContext
+            If called outside of an EventsContext
+
+        Examples
+        --------
+        Basic event sending:
+
+        >>> from haiway import ctx, State
+        >>>
+        >>> class OrderCreated(State):
+        ...     order_id: str
+        ...     amount: float
+        >>>
+        >>> async def process_order():
+        ...     # Send event after order creation
+        ...     ctx.send(OrderCreated(order_id="12345", amount=99.99))
+
+        See Also
+        --------
+        ctx.subscribe : For subscribing to events
+        """
+        EventsContext.send(payload)
+
+    @staticmethod
+    def subscribe[Payload: State](
+        payload_type: type[Payload],
+    ) -> EventSubscription[Payload]:
+        """
+        Subscribe to events of a specific type within the current context.
+
+        Creates a subscription that receives all events of the specified type
+        sent after the subscription is created.
+
+        Parameters
+        ----------
+        payload_type : type[Payload]
+            The State type to subscribe to. Must be a State class.
+
+        Returns
+        -------
+        EventSubscription[Payload]
+            An async iterator that yields events of the specified type
+
+        Raises
+        ------
+        MissingContext
+            If called outside of an EventsContext
+
+        Examples
+        --------
+        Basic subscription:
+
+        >>> from haiway import ctx, State
+        >>>
+        >>> class UserActivity(State):
+        ...     user_id: str
+        ...     action: str
+        ...     timestamp: float
+        >>>
+        >>> async def monitor_activity():
+        ...     async for activity in ctx.subscribe(UserActivity):
+        ...         print(f"{activity.user_id} performed {activity.action}")
+
+        With error handling:
+
+        >>> async def process_events():
+        ...     try:
+        ...         async for event in ctx.subscribe(PaymentEvent):
+        ...             await handle_payment(event)
+        ...     except asyncio.CancelledError:
+        ...         ctx.log_info("Payment processing stopped")
+        ...         raise
+
+        See Also
+        --------
+        ctx.send : For sending events
+        EventSubscription : The subscription iterator
+        """
+        return EventsContext.subscribe(payload_type)
+
+    @overload
+    @staticmethod
+    def variable(
+        variable: State,
+        /,
+    ) -> None: ...
+
+    @overload
+    @staticmethod
+    def variable[Variable: State](
+        variable: type[Variable],
+        /,
+    ) -> Variable | None: ...
+
+    @overload
+    @staticmethod
+    def variable[Variable: State](
+        variable: type[Variable],
+        /,
+        *,
+        default: Variable,
+    ) -> Variable: ...
+
+    @staticmethod
+    def variable[Variable: State](
+        variable: type[Variable] | State,
+        /,
+        *,
+        default: Variable | None = None,
+    ) -> Variable | None:
+        """
+        Get or set a mutable context-local variable in the current scope.
+
+        Context variables provide a way to store and retrieve mutable state that is
+        strictly local to the current execution context. Unlike regular state (accessed
+        via ctx.state()), context variables are mutable within their scope and propagate
+        changes to parent scopes when the current scope exits.
+
+        This method has two modes:
+        - **Set mode**: Pass a State instance to set/update a variable
+        - **Get mode**: Pass a State type to retrieve a variable
+
+        Important: Variables are NOT inherited from parent scopes. Each scope starts
+        with no variables unless explicitly set. This prevents issues with stale data
+        and ensures predictable behavior.
+
+        Parameters
+        ----------
+        variable : type[Variable] | State
+            Either a State type to retrieve a variable, or a State instance to set
+        default : Variable | None, optional
+            Default value to return if the variable is not set in the current scope.
+            Only used in get mode. If None (default), returns None when variable is not found.
+
+        Returns
+        -------
+        Variable | None
+            In get mode: The current value of the context variable or default/None if not set
+            In set mode: None (returns nothing)
+
+        Raises
+        ------
+        MissingContext
+            If called outside of any scope context
+
+        Examples
+        --------
+        Basic usage:
+
+        >>> from haiway import ctx, State
+        >>>
+        >>> class Counter(State):
+        ...     value: int = 0
+        >>>
+        >>> async def track_operations():
+        ...     # Set a variable (pass instance)
+        ...     ctx.variable(Counter(value=0))
+        ...
+        ...     # Get the variable (pass type)
+        ...     counter = ctx.variable(Counter)
+        ...     print(f"Current: {counter.value}")  # Current: 0
+        ...
+        ...     # Update the variable
+        ...     ctx.variable(counter.updated(value=counter.value + 1))
+
+        Using defaults:
+
+        >>> async def increment():
+        ...     # Get with default if not set
+        ...     counter = ctx.variable(Counter, default=Counter())
+        ...
+        ...     # Increment and update
+        ...     ctx.variable(counter.updated(value=counter.value + 1))
+
+        Scope isolation and propagation:
+
+        >>> async def outer():
+        ...     ctx.variable(Counter(value=10))  # Set in parent
+        ...
+        ...     async with ctx.scope("inner"):
+        ...         # Inner scope does NOT inherit parent's variable
+        ...         counter = ctx.variable(Counter)  # None - not set in inner
+        ...
+        ...         # Set variable in inner scope
+        ...         ctx.variable(Counter(value=20))
+        ...
+        ...     # After inner scope exits, inner's variables propagate to parent
+        ...     counter = ctx.variable(Counter)  # value = 20 (overwritten by inner)
+
+        Task isolation:
+
+        >>> async def worker():
+        ...     # Spawned tasks have isolated variable contexts
+        ...     counter = ctx.variable(Counter)  # Always None in spawned tasks
+        ...     ctx.variable(Counter(value=100))  # Won't affect parent
+        >>>
+        >>> async def main():
+        ...     ctx.variable(Counter(value=0))  # Set in main
+        ...     await ctx.spawn(worker)  # Task runs with isolated variables
+        ...     counter = ctx.variable(Counter)  # Still 0, unaffected by task
+
+        See Also
+        --------
+        ctx.state : For immutable state that doesn't propagate changes
+        """
+        if isinstance(variable, type):
+            current: Variable | None = VariablesContext.get(variable)
+            if current is None:
+                return default
+
+            else:
+                return current
+
+        else:
+            assert default is None  # nosec: B101
+            VariablesContext.set(variable)
+
     @staticmethod
     def log_error(
         message: str,
@@ -603,8 +1066,9 @@ class ctx:
         **extra: Any,
     ) -> None:
         """
-        Log using ERROR level within current scope context. When there is no current scope\
-         root logger will be used without additional details.
+        Log using ERROR level within current scope context.
+
+        When there is no current scope, root logger will be used without additional details.
 
         Parameters
         ----------
@@ -639,8 +1103,9 @@ class ctx:
         **extra: Any,
     ) -> None:
         """
-        Log using WARNING level within current scope context. When there is no current scope\
-         root logger will be used without additional details.
+        Log using WARNING level within current scope context.
+
+        When there is no current scope, root logger will be used without additional details.
 
         Parameters
         ----------
@@ -674,8 +1139,9 @@ class ctx:
         **extra: Any,
     ) -> None:
         """
-        Log using INFO level within current scope context. When there is no current scope\
-         root logger will be used without additional details.
+        Log using INFO level within current scope context.
+
+        When there is no current scope, root logger will be used without additional details.
 
         Parameters
         ----------
@@ -707,8 +1173,9 @@ class ctx:
         **extra: Any,
     ) -> None:
         """
-        Log using DEBUG level within current scope context. When there is no current scope\
-         root logger will be used without additional details.
+        Log using DEBUG level within current scope context.
+
+        When there is no current scope, root logger will be used without additional details.
 
         Parameters
         ----------
@@ -782,6 +1249,7 @@ class ctx:
         metric: str,
         value: float | int,
         unit: str | None = None,
+        kind: ObservabilityMetricKind,
         attributes: Mapping[str, ObservabilityAttribute] | None = None,
     ) -> None: ...
 
@@ -794,6 +1262,7 @@ class ctx:
         metric: str | None = None,
         value: float | int | None = None,
         unit: str | None = None,
+        kind: ObservabilityMetricKind | None = None,
         attributes: Mapping[str, ObservabilityAttribute] | None = None,
     ) -> None:
         if event is not None:
@@ -807,11 +1276,13 @@ class ctx:
         elif metric is not None:
             assert event is None  # nosec: B101
             assert value is not None  # nosec: B101
+            assert kind is not None  # nosec: B101
             ObservabilityContext.record_metric(
                 level,
                 metric,
                 value=value,
                 unit=unit,
+                kind=kind,
                 attributes=attributes or {},
             )
 
