@@ -6,11 +6,19 @@ from haiway.helpers.concurrent import (
     stream_concurrently,
 )
 from haiway.helpers.files import File, FileAccess
+from haiway.helpers.http_client import (
+    HTTPClient,
+    HTTPClientError,
+    HTTPHeaders,
+    HTTPQueryParams,
+    HTTPRequesting,
+    HTTPResponse,
+    HTTPStatusCode,
+)
 from haiway.helpers.observability import LoggerObservability
 from haiway.helpers.retries import retry
 from haiway.helpers.throttling import throttle
 from haiway.helpers.timeouting import timeout
-from haiway.helpers.tracing import traced
 
 __all__ = (
     "CacheMakeKey",
@@ -18,6 +26,13 @@ __all__ = (
     "CacheWrite",
     "File",
     "FileAccess",
+    "HTTPClient",
+    "HTTPClientError",
+    "HTTPHeaders",
+    "HTTPQueryParams",
+    "HTTPRequesting",
+    "HTTPResponse",
+    "HTTPStatusCode",
     "LoggerObservability",
     "asynchronous",
     "cache",
@@ -27,5 +42,4 @@ __all__ = (
     "stream_concurrently",
     "throttle",
     "timeout",
-    "traced",
 )
